// VECT_TAB_SRAM
// The symbol VECT_TAB_SRAM needs to be defined when building the project
// if code has been located to RAM and interrupts are used.

#include <stm32f10x.h>

#include "ndelay.h"
#include "dynamic_display.h"

//  .
volatile uint32_t tick_counter=0;

//     .
extern "C" void SysTick_Handler()
{
	if(tick_counter&1)
		display_refresh();

	tick_counter++;
}

//     
// (  ,     ).
void delay(uint32_t ms)
{
	uint32_t t0=tick_counter;
	while(tick_counter-t0<ms);
}

//      .
//      PB0.

//    1.
inline void set_dq1()
{
	GPIOB->BSRR=GPIO_BSRR_BS0;
}

//    0.
inline void set_dq0()
{
	GPIOB->BRR=GPIO_BRR_BR0;
}

//      (0 - false  1-true).
inline bool get_dq()
{
	return GPIOB->IDR&GPIO_IDR_IDR0;
}

// :       .
//  true,      .
bool reset()
{
	const uint32_t RESET_TIME_LOW=500;
	const uint32_t RESET_TIME_HIGH=500;
	const uint32_t PRESENCE_DETECT_POINT=68;

	//      .
	bool primask=__get_PRIMASK();
	__disable_irq();

	set_dq0();        //  .
	ndelay(USTON(RESET_TIME_LOW));
	set_dq1();        //  .
	ndelay(USTON(PRESENCE_DETECT_POINT));
	bool res=!get_dq();

	//    
	// :    
	//     .
	__set_PRIMASK(primask);

	ndelay(USTON(RESET_TIME_HIGH-PRESENCE_DETECT_POINT));
	return res;
}

//    0.
inline void slot_w0()
{
	set_dq0();
	ndelay(USTON(90));
	set_dq1();
}

//    1.
inline void slot_w1()
{
	set_dq0();
	ndelay(USTON(7));
	set_dq1();
	ndelay(USTON(83));
}

//   +   .
void slot_w(bool bit)
{
	bool primask=__get_PRIMASK();
	__disable_irq();

	if(bit)
		slot_w1();
	else
		slot_w0();

	__set_PRIMASK(primask);

	ndelay(USTON(10));
}

//     .
bool slot_r()
{
	bool primask=__get_PRIMASK();
	__disable_irq();

	set_dq0();
	ndelay(USTON(6));
	set_dq1();
	ndelay(USTON(6));
	bool res=get_dq();

	__set_PRIMASK(primask);

	ndelay(USTON(90));
	return res;
}

//   .
void send_byte(uint8_t b)
{
	for(uint32_t i=0; i<8; i++)
	{
		slot_w(b&1);
		b>>=1;
	}
}

//    .
uint8_t receive_byte()
{
	uint8_t res=0;
	for(uint32_t i=0; i<8; i++)
	{
		res>>=1;
		res|=slot_r()<<7;
	}
	return res;
}

int16_t get_themp_x16()
{
	int16_t t=0x8000;
	if(reset())
	{
		send_byte(0xCC);    // Skip ROM.
		send_byte(0x44);    // Convert T.
		while(slot_r()==0)  //  .
			delay(75);      // ,    .
		if(reset())
		{
			send_byte(0xCC);      // Skip ROM.
			send_byte(0xBE);      // Read scratchpad.
			t=receive_byte();     //   .
			t|=receive_byte()<<8; //   .
		}
	}
	return t;
}

int main()
{
	//     .
	display_config();

	//     .
	// PB1 - out (power); PB0 - in/out (open drain).
	RCC->APB2ENR|=RCC_APB2ENR_IOPBEN;

	const uint32_t mask=GPIO_CRL_CNF0|GPIO_CRL_MODE0|
			GPIO_CRL_CNF1|GPIO_CRL_MODE1;
	GPIOB->CRL=GPIOB->CRL&~mask|
			(GPIO_CRL_MODE0_1|GPIO_CRL_CNF0_0|
			GPIO_CRL_MODE1_1);
	// DS18B20 Power on:
	GPIOB->BSRR=GPIO_BSRR_BS1;

	SysTick_Config(SystemCoreClock/1000);

	while(true)
	{
		int tx16=get_themp_x16();
		int D=tx16*100/16;
		if(D<0)
			D-=5;
		else
			D+=5;
		D/=10;
		display(D, 1);
		delay(500);
	}
}
